<?php

/* * ******************************************
 * LiteSpeed Web Server Plugin for DirectAdmin
 *
 * @author LiteSpeed Technologies, Inc. (https://www.litespeedtech.com)
 * @copyright (c) 2019
 * @since 2.0
 * ******************************************* */

namespace LsPanel;

use \Lsc\Wp\LSCMException;
use \Lsc\Wp\Panel\ControlPanel;
use \LsPanel\DALiteSpeedControllerBase;
use \LsPanel\DALiteSpeedUtil;
use \LsPanel\DAPluginLogger;
use \LsPanel\View\Model as DAViewModel;

/**
 * @since 2.0
 */
class DALiteSpeedController extends DALiteSpeedControllerBase
{

    /**
     *
     * @since 2.0
     */
    public function __construct()
    {
        parent::__construct();
    }

    /**
     *
     * @since 2.0
     *
     * @return null
     */
    public function run()
    {
        $step = DALiteSpeedUtil::get_request_var('step', 0);
        $mgrStep = DALiteSpeedUtil::get_request_var('mgr_step', 0);
        $dashStep = DALiteSpeedUtil::get_request_var('dash_step', 0);

        $steps = array ( 0, 1, 2, 3);
        $mgrSteps = array( 0, 1, 2, 3, 4);
        $dashSteps = array( 0, 1, 2);

        $do = DALiteSpeedUtil::get_request_var('do');

        $supportedActions = array (
            'restart_lsws', 'check_current_license', 'change_license', 'transfer_license',
            'lscwp_manage', 'cacheRootSetup', 'lscwp_mass_enable_disable', 'lscwpVersionManager',
            'dash_notifier', 'dash_slug_verify', 'updateSharedCode', 'main', ''
        );

        if ( !in_array($do, $supportedActions)
                || !in_array($step, $steps)
                || !in_array($mgrStep, $mgrSteps)
                || !in_array($dashStep, $dashSteps)) {

            echo '<h1>Invalid Entrance</h1>';
            return;
        }

        ob_start();

        switch ($do) {

            case 'restart_lsws':
                $viewModel = $this->restartLsws($step);
                break;

            case 'check_current_license':
                $viewModel = new DAViewModel\CurrentLicenseStatusViewModel();
                break;

            case 'change_license':
                $viewModel = $this->change_license($step);
                break;

            case 'transfer_license':
                $viewModel = new DAViewModel\TransferLicenseViewModel();
                break;

            case 'lscwp_manage':
            case 'cacheRootSetup':
            case 'lscwpVersionManager':
            case 'lscwp_mass_enable_disable':
            case 'dash_notifier':
                $viewModel = $this->doSharedCodeDependentAction($do, $step,
                        $mgrStep, $dashStep);
                break;

            case 'dash_slug_verify':
                $viewModel = $this->dashSlugVerify();
                break;

            case 'updateSharedCode':
                $viewModel = $this->updateSharedCode();
                break;

            case 'main':
            case '':
                $viewModel = $this->getMainMenuViewModel();
                break;

            //no default
        }

        if ( !empty($viewModel) ) {
            $this->display($viewModel);
        }

        if ( defined('LSWS_AJAX_CALL') && LSWS_AJAX_CALL != false ) {
            $this->handleDefaultAjaxResponse(ob_get_clean());
        }

        ob_flush();
    }

    /**
     *
     * @since 2.0
     *
     * @param string  $do
     * @param int     $step
     * @param int     $mgrStep
     * @param int     $dashStep
     * @return object  ViewModel object.
     */
    private function doSharedCodeDependentAction($do, $step, $mgrStep,
            $dashStep)
    {
        if ( ! SHARED_CODE_LOADED ) {
            return (new DAViewModel\SharedCodeUpdateRequiredViewModel());
        }

        if ( version_compare(ControlPanel::PANEL_API_VERSION, '1.9', '<') ) {
            return (new DAViewModel\SharedCodeUpdateRequiredViewModel());
        }

        $panelAPICompatStatus =
                ControlPanel::checkPanelAPICompatibility(self::SHARED_API_VER);

        switch ( $panelAPICompatStatus ) {

            case ControlPanel::PANEL_API_VERSION_TOO_HIGH:
                return (new DAViewModel\SharedCodeUpdateRequiredViewModel());

            case ControlPanel::PANEL_API_VERSION_TOO_LOW:
            case ControlPanel::PANEL_API_VERSION_UNKNOWN:
                return (new DAViewModel\PluginUpdateRequiredViewModel());

            //no default
        }

        switch ($do) {
            case 'lscwp_manage':
                $viewModel = $this->lscwpManage($mgrStep);
                break;

            case 'cacheRootSetup':
                $viewModel = $this->cacheRootSetup($step);
                break;

            case 'lscwpVersionManager':
                $viewModel = $this->lscwpVersionManager($step);
                break;

            case 'lscwp_mass_enable_disable':
                $viewModel = $this->lscwpMassEnableDisableControl($step);
                break;

            case 'dash_notifier':
                $viewModel = $this->dashNotifier($dashStep);
                break;

            //no default
        }

        return $viewModel;
    }

    /**
     *
     * @since 2.0
     *
     * @return object  ViewModel object.
     */
    protected function getMainMenuViewModel()
    {
        return (new DAViewModel\MainMenuViewModel());
    }

    /**
     *
     * @since 2.0
     *
     * @param int  $step
     * @return object  $viewModel  ViewModel object.
     */
    private function change_license( $step )
    {
        $info = array(
            'errors' => array(),
            'install_type' => '',
            'license_agree' => '',
            'serial_no' => ''
        );

        if ( $step == 1 ) {
            DALiteSpeedUtil::validateLicenseChangeInput($info);

            if ( empty($info['errors']) ) {
                DALiteSpeedUtil::switchLicense($info);

                return (new DAViewModel\ChangeLicenseViewModel($info));
            }
        }

        return (new DAViewModel\ChangeLicenseConfirmViewModel($info));
    }

    /**
     *
     * @since 2.0
     *
     * @param int  $step
     * @return CacheRootSetupViewModel
     */
    private function cacheRootSetup( $step )
    {
        $panelEnv = $this->getControlPanelInstance();

        if ( $step == 1 ) {

            try {
                $panelEnv->verifyCacheSetup();
            }
            catch ( LSCMException $e ) {
                $msg = $e->getMessage();
                DAPluginLogger::error($msg);
                DAPluginLogger::uiError($msg);
            }
        }

        return (new DAViewModel\CacheRootSetupViewModel($panelEnv));
    }

}
