<?php

/* * ******************************************
 * LiteSpeed Web Server Plugin for DirectAdmin
 *
 * @author LiteSpeed Technologies, Inc. (https://www.litespeedtech.com)
 * @copyright (c) 2019
 * @since 2.0
 * ******************************************* */

namespace LsPanel;

use \Lsc\Wp\Context\Context;
use \Lsc\Wp\LSCMException;
use \Lsc\Wp\Panel\ControlPanel;
use \Lsc\Wp\PluginVersion;
use \Lsc\Wp\View\Model as LscViewModel;
use \LsPanel\DALiteSpeedUtil;
use \LsPanel\DAPluginAjaxResponse;
use \LsPanel\DAPluginException;
use \LsPanel\DAPluginLogger;
use \LsPanel\Extend\DAPanelController;
use \LsPanel\Extend\DAPanelWPDashMsgs;
use \LsPanel\Extend\DAPanelWPInstallStorage;
use \LsPanel\View\Model as DAViewModel;
use \LsPanel\View\View;

/**
 * @since 2.0
 */
abstract class DALiteSpeedControllerBase
{

    const ADMIN_CGI_DIR = '/usr/local/directadmin/plugins/litespeed/admin';
    const MODULE_VERSION = '2.0.0.2';
    const SHARED_API_VER = '1.9';

    /**
     *
     * @since 2.0
     *
     * @throws DAPluginException
     */
    public function __construct()
    {
        /**
         * Fixed location in DA.
         */
        $lswsHome = '/usr/local/lsws';

        if ( is_executable("{$lswsHome}/bin/lshttpd") ) {
            define('LSWS_HOME', $lswsHome);

            if ( $this->includeWebCacheMgrBootstrap() ) {
                define('SHARED_CODE_LOADED', true);

                try {
                    Context::getoption()->setIconDir('./litespeed/images/icons');
                }
                catch ( LSCMException $e ) {
                    $msg = $e->getMessage() . ' Unable to set icon directory.';
                    DAPluginLogger::debug($msg);

                    throw new DAPluginException($msg);
                }
            }
        }

        if ( !defined('SHARED_CODE_LOADED') ) {
            define('SHARED_CODE_LOADED', false);
        }

        DALiteSpeedUtil::initialize();
    }

    /**
     *
     * @since 2.0
     *
     * @return null
     */
    abstract public function run();

    /**
     * Output's an Ajax response with a default 'Content-Type' of 'text/html'
     * when an Ajax request has been made and a response is not explicitly sent
     * elsewhere in the code.
     *
     * @since 2.0
     *
     * @param string  $ajaxContent
     */
    protected function handleDefaultAjaxResponse( $ajaxContent )
    {
        $header = "HTTP/1.1 200 OK\nContent-Type: text/html";
        DAPluginAjaxResponse::setHeaderContent($header);
        DAPluginAjaxResponse::setAjaxContent($ajaxContent);
        DAPluginAjaxResponse::outputAndExit();
    }

    /**
     *
     * @since 2.0
     *
     * @param string  $msg
     */
    protected function displayMissingTpl( $msg )
    {
        $viewModel = new DAViewModel\MissingTplViewModel($msg);
        $view = new View($viewModel);
        $view->display();
    }

    /**
     * Attempt to include shared code.
     *
     * @since 2.0
     *
     * @return boolean
     */
    protected function includeWebCacheMgrBootstrap()
    {
        if ( ! file_exists(LSWS_HOME . '/add-ons/webcachemgr/autoloader.php')
                || ! include_once __DIR__ . '/webCacheMgrBootstrap.php' ) {

            return false;
        }

        return true;
    }

    /**
     *
     * @since 2.0
     *
     * @return object  ViewModel object.
     */
    protected function updateSharedCode()
    {
        $updated = DALiteSpeedUtil::updateSharedCode();

        return (new DAViewModel\SharedCodeUpdateResultViewModel($updated));
    }

    /**
     *
     * @since 2.0
     *
     * @return object  ViewModel object.
     */
    abstract protected function getMainMenuViewModel();


    /**
     *
     * @since 2.0
     *
     * @param int  $step
     * @return object  $viewModel  ViewModel object.
     */
    protected function restartLsws( $step )
    {
        if ( $step == 1 ) {
            DALiteSpeedUtil::restartLSWS();

            $viewModel = new DAViewModel\RestartLswsViewModel();
        }
        else {
            $viewModel = new DAViewModel\RestartLswsConfirmViewModel();
        }

        return $viewModel;
    }

    /**
     *
     * @since 2.0
     *
     * @return \LsPanel\Extend\DirectAdminPanel
     */
    protected function getControlPanelInstance()
    {
        return ControlPanel::getClassInstance('\LsPanel\Extend\DirectAdminPanel');
    }

    /**
     *
     * @since 2.0
     *
     * @param int  $mgrStep
     * @return object  ViewModel object.
     */
    protected function lscwpManage( $mgrStep )
    {
        $panelEnv = $this->getControlPanelInstance();

        if ( !$panelEnv->areCacheRootsSet() ) {
            return (new LscViewModel\CacheRootNotSetViewModel());
        }

        $lscmDataFiles = Context::getLSCMDataFiles();

        $model = new DAPanelWPInstallStorage($lscmDataFiles['dataFile'],
                $lscmDataFiles['custDataFile']);

        $panelController =
                new DAPanelController($panelEnv, $model, $mgrStep);

        /**
         * Pre-initialize DAPanelPluginVersion instance for viewModel.
         */
        PluginVersion::getInstance();

        $newMgrStep = $panelController->manageCacheOperations();

        switch ($newMgrStep) {
            case DAPanelController::MGR_STEP_SCAN:
            case DAPanelController::MGR_STEP_DISCOVER_NEW:
                return (new LscViewModel\ScanProgressViewModel($newMgrStep));
            case DAPanelController::MGR_STEP_REFRESH_STATUS:
                return (new LscViewModel\RefreshStatusProgressViewModel());
            case DAPanelController::MGR_STEP_MASS_UNFLAG:
                return (new LscViewModel\UnflagAllProgressViewModel());
            default:
                return (new LscViewModel\ManageViewModel($model));
        }
    }

    /**
     *
     * @since 2.0
     *
     * @param int  $step
     * @return object  ViewModel object.
     */
    protected function lscwpMassEnableDisableControl( $step )
    {
        $lscmDataFiles = Context::getLSCMDataFiles();
        $model = new DAPanelWPInstallStorage($lscmDataFiles['dataFile'],
                $lscmDataFiles['custDataFile']);

        if ( $step == 1 || $step == 2 ) {
            $allowedActions = array( 'enable', 'disable' );

            $action = DALiteSpeedUtil::get_request_var('act');

            if ( in_array($action, $allowedActions) ) {
                return $this->lscwpMassEnableDisable($action, $step, $model);
            }
        }
        elseif ( $step == 0 ) {

            if ( $model->getError() != 0 ) {
                return (new LscViewModel\DataFileMsgViewModel($model));
            }

            $panelEnv = $this->getControlPanelInstance();

            if ( !$panelEnv->areCacheRootsSet() ) {
                return (new LscViewModel\CacheRootNotSetViewModel());
            }
        }

        return (new LscViewModel\MassEnableDisableViewModel($model));
    }

    /**
     *
     * @since 2.0
     *
     * @param string                      $action
     * @param int                         $step
     * @param PleskPanelWPInstallStorage  $model
     * @return LscViewModel\MassEnableDisableProgressViewModel|null
     */
    protected function lscwpMassEnableDisable( $action, $step,
            DAPanelWPInstallStorage $model )
    {
        if ( $model->getError() != 0 ) {
            return $this->getMainMenuViewModel();
        }

        $panelEnv = $this->getControlPanelInstance();
        $panelController = new DAPanelController($panelEnv, $model);

        $panelController->massEnableDisable($action, $step);

        $viewModel =
                new LscViewModel\MassEnableDisableProgressViewModel($action);

        return $viewModel;
    }

    /**
     *
     * @param int  $step
     * @return object  View model object.
     */
    protected function lscwpVersionManager( $step )
    {
        $lscmDataFiles = Context::getLSCMDataFiles();
        $model = new DAPanelWPInstallStorage($lscmDataFiles['dataFile'],
                $lscmDataFiles['custDataFile']);

        if ( $step == 2 || $step == 3 ) {
            $action = DALiteSpeedUtil::get_request_var('act');

            switch ($action) {
                case 'switchTo':
                    $ver_num = DALiteSpeedUtil::get_request_var('version_num');

                    try {
                        PluginVersion::getInstance()->setActiveVersion($ver_num);
                    }
                    catch ( LSCMException $e ) {
                        $msg = $e->getMessage()
                                . " Could not switch active version to v{$ver_num}";

                        DAPluginLogger::error($msg);
                        DAPluginLogger::uiError($msg);
                    }

                    return (new LscViewModel\VersionManageViewModel($model));

                case 'upgradeTo':
                    $panelEnv = $this->getControlPanelInstance();
                    $panelController = new DAPanelController($panelEnv, $model);

                    $panelController->prepVersionChange($step);

                    return (new LscViewModel\VersionChangeViewModel());
            }
        }

        return (new LscViewModel\VersionManageViewModel($model));
    }

    /**
     *
     * @since 2.0
     *
     * @param int  $dashStep
     * @return object  $viewModel  ViewModel object.
     */
    protected function dashNotifier( $dashStep )
    {
        $panelEnv = $this->getControlPanelInstance();

        $lscmDataFiles = Context::getLSCMDataFiles();
        $wpInstallStorage = new DAPanelWPInstallStorage($lscmDataFiles['dataFile'],
                $lscmDataFiles['custDataFile']);

        $panelController = new DAPanelController($panelEnv, $wpInstallStorage);
        $wpDashMsgs = new DAPanelWPDashMsgs();

        $dashStep =
                $panelController->manageDashOperations($dashStep, $wpDashMsgs);

        switch ( $dashStep ) {
            case DAPanelController::DASH_STEP_MASS_DASH_NOTIFY:
                $viewModel = new LscViewModel\MassDashNotifyProgressViewModel();
                break;
            case DAPanelController::DASH_STEP_MASS_DASH_DISABLE:
                $viewModel =
                    new LscViewModel\MassDashDisableProgressViewModel();
                break;
            default:
                $viewModel = new LscViewModel\DashNotifierViewModel(
                        $wpInstallStorage, $wpDashMsgs);
        }

        return $viewModel;
    }

    /**
     * Handle slug verification server side to bypass same domain policy
     * restrictions.
     *
     * @since 2.0
     */
    protected function dashSlugVerify()
    {
        $slug = DALiteSpeedUtil::get_request_var('slug');

        if ( $slug == NULL ) {
            $jsonData = '';
        }
        else {
            $url = filter_var("https://api.wordpress.org/plugins/info/1.0/{$slug}.json",
                    FILTER_SANITIZE_URL);

            /**
             * $jsonData should already be in JSON format.
             */
            $jsonData = DALiteSpeedUtil::getUrlContents($url);
        }

        $header = "HTTP/1.1 200 OK\nContent-Type: application/json";
        DAPluginAjaxResponse::setHeaderContent($header);
        DAPluginAjaxResponse::setAjaxContent($jsonData);
        DAPluginAjaxResponse::outputAndExit();
    }

    /**
     * Creates a View object with the passes ViewModel and displays to screen.
     *
     * @since 2.0
     *
     * @param object  $viewModel  ViewModel object.
     */
    protected function display( $viewModel )
    {
        $view = new View($viewModel);

        try {
            $view->display();
        }
        catch ( DAPluginException $e ) {
            $this->displayMissingTpl($e->getMessage());
        }
    }

}
