<?php

/* * ******************************************
 * LiteSpeed Web Server Plugin for DirectAdmin
 *
 * @author LiteSpeed Technologies, Inc. (https://www.litespeedtech.com)
 * @copyright (c) 2019
 * @since 2.0
 * ******************************************* */

namespace LsPanel;

use \LsPanel\DAEscalate;
use \LsPanel\DAPluginException;

/**
 * @since 2.0
 */
class DALiteSpeedUtil
{

    /**
     * @since 2.0
     * @var null|DALiteSpeedUtil
     */
    private static $instance;

    /**
     *
     * @since 2.0
     */
    private function __construct()
    {

    }

    /**
     *
     * @since 2.0
     *
     * @throws DAPluginException
     */
    public static function initialize()
    {
        if ( self::$instance != null ) {
            throw new DAPluginException('DALiteSpeedUtil cannot be initialized twice.',
                    DAPluginException::E_PROGRAM);
        }

        self::$instance = new static();
    }

    /**
     *
     * @since 2.0
     *
     * @param string  $tag
     * @param mixed   $default  The default value to be passed back when
     *                          the request var is not found.
     * @return mixed|null|string
     */
    public static function get_request_var( $tag, $default = null )
    {
        $varValue = $default;

        if ( $_SERVER['REQUEST_METHOD'] === 'POST' ) {
            $querystring = urldecode(getenv('POST'));
        }
        else {
            $querystring = urldecode(getenv('QUERY_STRING'));
        }

        if ( $querystring != ''
                && preg_match("/(?:^|\?|&){$tag}=([^&]+)/", $querystring, $m) ) {

            $varValue = $m[1];

            if ( $varValue != null ) {
                $varValue = trim($varValue);
            }
        }

        return $varValue;
    }

    /**
     * Used as a wrapper function for exec() calls.
     *
     * @since 2.0
     *
     * @param string         $cmd
     * @param null|string[]  $output
     * @param null|int       $ret
     */
    public static function exec_cmd( $cmd, array &$output = null,
            &$ret = null )
    {
        exec($cmd, $output, $ret);
    }

    /**
     * Used as a wrapper for shell_exec() calls.
     *
     * @since 2.0
     *
     * @param string  $cmd
     * @return null|string
     */
    public static function shell_exec_cmd( $cmd )
    {
        $output = shell_exec($cmd);
        return $output;
    }

    /**
     * Returns LiteSpeed Web Server process ID, or 0 if it is not running.
     *
     * @since 2.0
     *
     * @return int
     */
    public static function getLSPID()
    {
        $pidfile = '/tmp/lshttpd/lshttpd.pid';

        if ( file_exists($pidfile) ) {
            $pid = trim(file_get_contents($pidfile));

            if ( $pid == '' ) {
                return 0;
            }

            posix_kill($pid, 0);

            if ( posix_get_last_error() == 1 ) { /* EPERM */
                return $pid;
            }
        }

        return 0;
    }

    /**
     * Returns Apache process ID, or 0 if it is not running.
     *
     * @since 2.0
     *
     * @return int
     */
    public static function getApachePID()
    {
        $pidfile = '/var/run/httpd.pid';

        if ( file_exists($pidfile) ) {
            $pid = trim(file_get_contents($pidfile));

            if ( $pid == '' ) {
                return 0;
            }

            posix_kill($pid, 0);

            if ( posix_get_last_error() == 1 ) { /* EPERM */
                return $pid;
            }
        }

        return 0;
    }

    /**
     *
     * @since 2.0
     */
    public static function restartLSWS()
    {
        DAEscalate::escalate(true, DAEscalate::LSWS_CMD_RESTART_LSWS);
    }

    /**
     *
     * @since 2.0
     *
     * @param string  $file
     * @param int     $pid
     * @return boolean
     */
    public static function pidBackup( $file, $pid )
    {
        return rename($file, "{$file}.bak.{$pid}");
    }

    /**
     *
     * @since 2.0
     *
     * @param int     $pid
     * @param string  $file
     */
    public static function undoPidBackup( $pid, $file )
    {
        rename("{$file}.bak.{$pid}", $file);
    }

    /**
     *
     * @since 2.0
     *
     * @return string[]
     */
    public static function getMainMenuInfo()
    {
        $args = array( 'serv_type' => LSWS_SERVER_TYPE );
        $result =
                DAEscalate::escalate(false,
                        DAEscalate::LSWS_CMD_GET_MAINMENU_INFO, $args);

        return $result;
    }

    /**
     *
     * @since 2.0
     *
     * @param string[]  $output
     * @return int
     */
    public static function getCurrentLicenseStatus( &$output )
    {
        $result = DAEscalate::escalate(true,
                DAEscalate::LSWS_CMD_GET_LICENSE_STATUS);

        if ( isset($result['status']) && $result['status'] != '' ) {
            $output .= "<p>{$result['status']}</p>\n";
        }

        if ( isset($result['MESG']) && $result['MESG'] != '' ) {
            $output .= "<p>{$result['MESG']}</p>\n";
        }

        return $result['return_var'];
    }

    /**
     *
     * @since 2.0
     *
     * @param string  $serial_no
     * @return string
     */
    public static function validate_serial_no( $serial_no )
    {
        $error = '';

        if ( $serial_no == "" ) {
            $error = 'Missing serial number!';
        }
        else {
            $pattern = '[a-zA-Z0-9/+]{4}';

            $validSerial = "%^{$pattern}-{$pattern}-{$pattern}-{$pattern}$%";

            if ( strlen($serial_no) != 19
                    || !preg_match($validSerial, $serial_no) ) {

                $error = 'Invalid serial number';
            }
        }

        return $error;
    }

    /**
     *
     * @since 2.0
     *
     * @param string[][]  $info
     */
    public static function validateLicenseChangeInput( &$info )
    {
        $errors = &$info['errors'];

        $info['license_agree'] = self::get_request_var('license_agree', '');

        if ( $info['license_agree'] != 'agree' ) {
            $errors['license_agree'] = 'Cannot proceed without agreement to EULA';
        }

        $info['install_type'] = self::get_request_var('install_type', '');

        switch ( $info['install_type'] ) {
            case 'trial':
                $info['serial_no'] = 'TRIAL';
                break;

            case 'prod':
                $serial_no = trim(self::get_request_var('serial_no', ''));
                $err = self::validate_serial_no($serial_no);

                if ( $err != "" ) {
                    $errors['serial_no'] = $err;
                }
                else {
                    $info['serial_no'] = $serial_no;
                }

                break;

            case '':
                $errors['install_type'] = 'Please select one';
                break;

            default:
                $errors['install_type'] = 'Invalid type';
        }
    }

    /**
     *
     * @since 2.0
     *
     * @param string[][]  $info
     * @return string
     */
    public static function switchLicense( &$info )
    {
        $info['output'] = '';

        $args = array ( 'serial_no' => $info['serial_no']);
        $result = DAEscalate::escalate(true,
                DAEscalate::LSWS_CMD_SWITCH_LICENSE, $args);

        if ( isset($result['status']) ) {
            $info['output'] .= $result['status'];
        }

        if ( !empty($result['MESG']) ) {
            $info['output'] .= "<p>{$result['MESG']}</p>\n";
        }

        return $result['return_var'];
    }

    /**
     *
     * @since 2.0
     *
     * @param string  $dir
     * @return false|string
     */
    public static function directoryMd5( $dir )
    {
        if ( !is_dir($dir) ) {
            return false;
        }

        $filemd5s = array();
        $d = dir($dir);

        while ( ($entry = $d->read()) !== false ) {

            if ( $entry != '.' && $entry != '..' ) {

                $currEntry = "{$dir}/{$entry}";

                if ( is_dir($currEntry) ) {
                    $filemd5s[] = self::directoryMd5($currEntry);
                }
                else {
                    $filemd5s[] = md5_file($currEntry);
                }
            }
        }

        $d->close();
        return md5(implode('', $filemd5s));
    }

    /**
     *
     * @since 2.0
     *
     * @param string[]  $output
     * @return string
     */
    public static function getOutputString( $output )
    {
        $buf = '';

        foreach ( $output as $o ) {

            if ( $o != '' ) {
                $buf .= "{$o} <br />";
            }
        }

        return $buf;
    }

    /**
     *
     * @since 2.0
     *
     * @param string   $url
     * @param boolean  $headerOnly
     * @return string
     */
    public static function getUrlContents( $url, $headerOnly = false )
    {
        if ( ini_get('allow_url_fopen') ) {
            /**
             * Silence warning when OpenSSL missing while getting LSCWP ver
             * file.
             */
            $url_content = @file_get_contents($url);

            if ( $url_content !== false ) {

                if ( $headerOnly ) {
                    return implode("\n", $http_response_header);
                }

                return $url_content;
            }
        }

        if ( function_exists('curl_version') ) {
            $ch = curl_init();

            curl_setopt_array(
                    $ch,
                    array(
                        CURLOPT_URL => $url,
                        CURLOPT_RETURNTRANSFER => true,
                        CURLOPT_HEADER => $headerOnly,
                        CURLOPT_NOBODY => $headerOnly,
                        CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1
                    )
            );

            $url_content = curl_exec($ch);
            curl_close($ch);

            if ( $url_content !== false ) {
                return $url_content;
            }
        }

        $cmd = 'curl';

        if ( $headerOnly ) {
            $cmd .= ' -s -I';
        }

        $url_content = exec("{$cmd} {$url}", $output, $ret);

        if ( $ret === 0 ) {
            $url_content = implode("\n", $output);
            return $url_content;
        }

        return '';
    }

    /**
     * Recursively remove a directory's contents and optionally the directory
     * itself. Intended to be used by DAEscalate only.
     *
     * @since 2.0
     *
     * @param string   $dir         Directory path
     * @param boolean  $keepParent  Only remove directory contents when true.
     */
    public static function e2_rrmdir( $dir, $keepParent = false )
    {
        if ( $dir != '' && is_dir($dir) ) {

            foreach ( glob("{$dir}/*") as $file ) {

                if ( is_dir($file) ) {
                    self::e2_rrmdir($file);
                }
                else {
                    unlink($file);
                }
            }

            if ( !$keepParent ) {
                rmdir($dir);
            }

            return true;
        }

        return false;
    }

    /**
     *
     * @since 2.0
     *
     * @return boolean
     */
    public static function updateSharedCode()
    {
        $ret = DAEscalate::escalate(true,
                DAEscalate::MGR_CMD_UPDATE_SHARED_CODE);

        return $ret;
    }

}
