<?php

/* * ******************************************
 * LiteSpeed Web Server Plugin for DirectAdmin
 *
 * @author LiteSpeed Technologies, Inc. (https://www.litespeedtech.com)
 * @copyright (c) 2019
 * @since 2.0
 * ******************************************* */

namespace LsPanel;

use \LsPanel\DAEscalate;
use \LsPanel\DAPluginLogEntry;
use \LsPanel\DAPluginException;

/**
 * DAPluginLogger is a singleton
 *
 * @since 2.0
 */
class DAPluginLogger
{

    /**
     * @since 2.0
     * @var string
     */
    const TEMP_LOG_BASE = '/tmp/ls-da-tmp';

    /**
     * @since 2.0
     * @var int
     */
    const L_NONE = 0;

    /**
     * @since 2.0
     * @var int
     */
    const L_ERROR = 1;

    /**
     * @since 2.0
     * @var int
     */
    const L_WARN = 2;

    /**
     * @since 2.0
     * @var int
     */
    const L_NOTICE = 3;

    /**
     * @since 2.0
     * @var int
     */
    const L_INFO = 4;

    /**
     * @since 2.0
     * @var int
     */
    const L_VERBOSE = 5;

    /**
     * @since 2.0
     * @var int
     */
    const L_DEBUG = 9;

    /**
     * @since 2.0
     * @var int
     */
    const UI_INFO = 0;

    /**
     * @since 2.0
     * @var int
     */
    const UI_SUCC = 1;

    /**
     * @since 2.0
     * @var int
     */
    const UI_ERR = 2;

    /**
     * @since 2.0
     * @var int
     */
    const UI_WARN = 3;

    /**
     * @since 2.0
     * @var null|DAPluginLogger
     */
    protected static $instance;

    /**
     * @since 2.0
     * @var int  Highest log message level allowed to be logged. Set to the
     *           higher value between $this->logFileLvl and $this->logEchoLvl.
     */
    protected $logLvl;

    /**
     * @since 2.0
     * @var string  File that log messages will be written to (if writable).
     */
    protected $logFile;

    /**
     * @since 2.0
     * @var int  Highest log message level allowed to be written to the log
     *           file.
     */
    protected $logFileLvl;

    /**
     * @since 2.0
     * @var string  Additional tag to be added at the start of any log
     *              messages.
     */
    protected $addTagInfo = '';

    /**
     * @since 2.0
     * @var boolean  When set to true, log messages will not be written to the
     *               log file immediately. Messages will be automatically
     *               written when this DAPluginLogger object is destroyed.
     */
    protected $bufferedWrite;

    /**
     * @since 2.0
     * @var DAPluginLogEntry[]  Stores created DAPluginLogEntry objects.
     */
    protected $msgQueue = array();

    /**
     * @since 2.0
     * @var int  Highest log message level allowed to echoed.
     */
    protected $logEchoLvl;

    /**
     * @since 2.0
     * @var boolean  When set to true, echoing of log messages will not occur
     *               immediately. Messages will be automatically echo'd when
     *               this DAPluginLogger object is destroyed.
     */
    protected $bufferedEcho;

    /**
     * @since 2.0
     * @var string[][]  Leveraged by Control Panel GUI to store and retrieve
     *                  display messages.
     */
    protected $uiMsgs = array(
        self::UI_INFO => array(),
        self::UI_SUCC => array(),
        self::UI_ERR => array(),
        self::UI_WARN => array()
    );

    /**
     *
     * @since 2.0
     */
    protected function __construct()
    {
        $this->init();
    }

    /**
     *
     * @since 2.0
     */
    protected function init()
    {
        date_default_timezone_set('UTC');

        $this->logFile = self::TEMP_LOG_BASE . '_' . getmypid() . '.log';
        $this->logFileLvl = self::L_INFO;
        $this->logEchoLvl = self::L_NONE;
        $this->bufferedWrite = true;
        $this->bufferedEcho = true;

        if ( $this->logEchoLvl >= $this->logFileLvl ) {
            $logLvl = $this->logEchoLvl;
        }
        else {
            $logLvl = $this->logFileLvl;
        }

        $this->logLvl = $logLvl;
    }

    /**
     *
     * @since 2.0
     */
    public function __destruct()
    {
        if ( $this->bufferedWrite ) {
            $this->writeToFile($this->msgQueue);
        }
    }

    /**
     *
     * @since 2.0
     *
     * @throws DAPluginException
     */
    public static function Initialize()
    {
        if ( self::$instance != null ) {
            throw new DAPluginException('Plugin logger class cannot be initialized twice.',
                    DAPluginException::E_PROGRAM);
        }

        self::$instance = new static();
    }

    /**
     * Create log file if it does not already exist. Useful for creating temp
     * log file with correct permissions before writing to it in escalated
     * functions.
     *
     * @since 2.0
     */
    public static function preCreateLogFile()
    {
        if ( !file_exists(self::me()->logFile) ) {
            touch(self::me()->logFile);
        }
    }

    /**
     *
     * @since 2.0
     *
     * @param string  $logFile
     */
    public static function changeLogFileUsed( $logFile )
    {
        self::me()->logFile = $logFile;
    }

    /**
     *
     * @since 2.0
     *
     * @param string  $addInfo
     */
    public static function setAdditionalTagInfo( $addInfo )
    {
        self::me()->addTagInfo = $addInfo;
    }

    /**
     *
     * @since 2.0
     *
     * @return string
     */
    public static function getAdditionalTagInfo()
    {
        return self::me()->addTagInfo;
    }

    /**
     *
     * @since 2.0
     *
     * @return string
     */
    public static function getLogFilePath()
    {
        return self::me()->logFile;
    }

    /**
     *
     * @since 2.0
     *
     * @param int  $type
     * @return string[]
     */
    public static function getUiMsgs( $type )
    {
        $ret = array();

        switch ($type) {
            case self::UI_INFO:
            case self::UI_SUCC:
            case self::UI_ERR:
            case self::UI_WARN:
                $ret = self::me()->uiMsgs[$type];
                break;

            //no default
        }

        return $ret;
    }

    /**
     * Processes any buffered output, writing it to the log file, echoing it
     * out, or both.
     *
     * @since 2.0
     */
    public static function processBuffer()
    {
        $clear = false;

        $m = self::me();

        if ( $m->bufferedWrite ) {
            $m->writeToFile($m->msgQueue);
            $clear = true;
        }

        if ( $m->bufferedEcho ) {
            $m->echoEntries($m->msgQueue);
            $clear = true;
        }

        if ( $clear ) {
            $m->msgQueue = array();
        }
    }

    /**
     *
     * @since 2.0
     *
     * @param string  $msg
     * @param int     $type
     */
    protected static function addUiMsg( $msg, $type )
    {
        switch ($type) {
            case self::UI_INFO:
            case self::UI_SUCC:
            case self::UI_ERR:
            case self::UI_WARN:
                self::me()->uiMsgs[$type][] = $msg;
                break;

            //no default
        }
    }

    /**
     * Calls addUiMsg() with message level self::UI_INFO.
     *
     * @since 2.0
     *
     * @param string  $msg
     */
    public static function uiInfo( $msg )
    {
        self::addUiMsg($msg, self::UI_INFO);
    }

    /**
     * Calls addUiMsg() with message level self::UI_SUCC.
     *
     * @since 2.0
     *
     * @param string  $msg
     */
    public static function uiSuccess( $msg )
    {
        self::addUiMsg($msg, self::UI_SUCC);
    }

    /**
     * Calls addUiMsg() with message level self::UI_ERR.
     *
     * @since 2.0
     *
     * @param string  $msg
     */
    public static function uiError( $msg )
    {
        self::addUiMsg($msg, self::UI_ERR);
    }

    /**
     * Calls addUiMsg() with message level self::UI_WARN.
     *
     * @since 2.0
     *
     * @param string  $msg
     */
    public static function uiWarning( $msg )
    {
        self::addUiMsg($msg, self::UI_WARN);
    }

    /**
     *
     * @since 2.0
     *
     * @param string  $msg
     * @param int     $lvl
     */
    protected static function logMsg( $msg, $lvl )
    {
        self::me()->log($msg, $lvl);
    }

    /**
     * Calls logMsg() with message level self::L_ERROR.
     *
     * @since 2.0
     *
     * @param string  $msg
     */
    public static function error( $msg )
    {
        self::logMsg($msg, self::L_ERROR);
    }

    /**
     * Calls logMsg() with message level self::L_WARN.
     *
     * @since 2.0
     *
     * @param string  $msg
     */
    public static function warn( $msg )
    {
        self::logMsg($msg, self::L_WARN);
    }

    /**
     * Calls logMsg() with message level self::L_NOTICE.
     *
     * @since 2.0
     *
     * @param string  $msg
     */
    public static function notice( $msg )
    {
        self::logMsg($msg, self::L_NOTICE);
    }

    /**
     * Calls logMsg() with message level self::L_INFO.
     *
     * @since 2.0
     *
     * @param string  $msg
     */
    public static function info( $msg )
    {
        self::logMsg($msg, self::L_INFO);
    }

    /**
     * Calls logMsg() with message level self::L_VERBOSE.
     *
     * @since 2.0
     *
     * @param string  $msg
     */
    public static function verbose( $msg )
    {
        self::logMsg($msg, self::L_VERBOSE);
    }

    /**
     * Calls logMsg() with message level self::L_DEBUG.
     *
     * @since 2.0
     *
     * @param string  $msg
     */
    public static function debug( $msg )
    {
        self::logMsg($msg, self::L_DEBUG);
    }

    /**
     *
     * @since 2.0
     *
     * @return DAPluginLogger
     * @throws DAPluginException
     */
    protected static function me()
    {
        if ( self::$instance == null ) {
            throw new DAPluginException('Plugin logger class uninitialized.',
                    DAPluginException::E_PROGRAM);
        }

        return self::$instance;
    }

    /**
     *
     * @since 2.0
     *
     * @param string  $mesg
     * @param int     $lvl
     */
    protected function log( $mesg, $lvl )
    {
        $entry = new DAPluginLogEntry($mesg, $lvl);

        $this->msgQueue[] = $entry;

        if ( !$this->bufferedWrite ) {
            $this->writeToFile(array( $entry ));
        }

        if ( !$this->bufferedEcho ) {
            $this->echoEntries(array( $entry ));
        }
    }

    /**
     * Escalate and write temp log file contents to real shared log file.
     *
     * @since 2.0
     *
     * @param DAPluginLogEntry[]  $entries
     */
    protected function writeToFile( $entries )
    {
        $content = '';

        foreach ( $entries as $e ) {
            $content .= $e->getOutput($this->logFileLvl);
        }

        if ( $content != '' ) {

            if ( $this->logFile ) {
                file_put_contents($this->logFile, $content,
                        FILE_APPEND | LOCK_EX);
            }
            else {
                error_log($content);
            }

            DAEscalate::escalate(true, DAEscalate::MGR_CMD_WRITE_TO_LOG);
        }

        if ( file_exists($this->logFile) ) {
            unlink($this->logFile);
        }
    }

    /**
     *
     * @since 2.0
     *
     * @param DAPluginLogEntry[]  $entries
     */
    protected function echoEntries( $entries )
    {
        foreach ( $entries as $entry ) {

            if ( ($msg = $entry->getOutput($this->logEchoLvl)) !== '' ) {
                echo $msg;
            }
        }
    }

    /**
     *
     * @since 2.0
     *
     * @param int  $lvl
     * @return string
     */
    public static function getLvlDescr( $lvl )
    {
        switch ($lvl) {
            case self::L_ERROR:
                return 'ERROR';
            case self::L_WARN:
                return 'WARN';
            case self::L_NOTICE:
                return 'NOTICE';
            case self::L_INFO:
                return 'INFO';
            case self::L_VERBOSE:
                return 'DETAIL';
            case self::L_DEBUG:
                return 'DEBUG';
            default:
                /**
                 * Do silently.
                 */
                return '';
        }
    }

    /**
     * Prevent cloning here and in extending classes.
     *
     * @since 2.0
     */
    final protected function __clone() {}

}
