<?php

/* * ******************************************
 * LiteSpeed Web Server Plugin for DirectAdmin
 *
 * @author LiteSpeed Technologies, Inc. (https://www.litespeedtech.com)
 * @copyright (c) 2019
 * @since 2.0
 * ******************************************* */

namespace LsPanel\Extend;

use \Lsc\Wp\Logger;
use \Lsc\Wp\Util;
use \Lsc\Wp\WPInstallStorage;
use \LsPanel\DAEscalate;
use \LsPanel\Extend\DAPanelWPInstall;

/**
 * map to data file
 *
 * @since 2.0
 */
class DAPanelWPInstallStorage extends WPInstallStorage
{

    /**
     *
     * @since 2.0
     *
     * @param string  $dataFile
     * @param string  $custDataFile
     */
    public function __construct( $dataFile, $custDataFile = '' )
    {
        parent::__construct($dataFile, $custDataFile);
    }

    /**
     *
     * @since 2.0
     *
     * @return int
     */
    protected function init()
    {
        $dataExists = false;

        try {
            $args = array ( 'filePath' => $this->dataFile );
            $dataFileExists = DAEscalate::escalate(true,
                    DAEscalate::MGR_CMD_DATA_FILE_EXISTS, $args);

            if ( $dataFileExists ) {
                $dataExists = true;
                $this->wpInstalls = $this->getDataFileData($this->dataFile);
            }

            if ( $this->customDataFile != '' ) {
                $args = array ( 'filePath' => $this->customDataFile );
                $custDataFileExists = DAEscalate::escalate(true,
                        DAEscalate::MGR_CMD_DATA_FILE_EXISTS, $args);

                if ( $custDataFileExists ) {
                    $dataExists = true;
                    $this->custWpInstalls =
                        $this->getDataFileData($this->customDataFile);
                }
            }
        }
        catch ( LSCMException $e ) {
            Logger::debug($e->getMessage());
            return $e->getCode();
        }

        if ( !$dataExists ) {
            return self::ERR_NOT_EXIST;
        }

        return 0;
    }

    /**
     *
     * @since 2.0
     *
     * @param string  $dataFile
     * @return DAPanelWPInstall[]
     * @throws LSCMException
     */
    protected function getDataFileData( $dataFile )
    {
        $args = array( 'filePath' => $dataFile );
        $content = DAEscalate::escalate(true,
                DAEscalate::MGR_CMD_GET_DATA_FILE_CONTENT, $args);

        if ( ($data = json_decode($content, true)) === null ) {
            /*
             * Data file may be in old serialized format. Try unserializing.
             */
             $data = unserialize($content);
        }

        if ( $data === false || !is_array($data) || !isset($data['__VER__']) ) {
            throw new LSCMException("{$dataFile} - Data is corrupt.",
                    self::ERR_CORRUPTED);
        }

        if ( ($err = $this->verifyDataFileVer($dataFile, $data['__VER__'])) ) {
            throw new LSCMException("{$dataFile} - Data file version issue.", $err);
        }

        unset($data['__VER__']);

        $wpInstalls = array();

        foreach ( $data as $path => $idata ) {
            $i = new DAPanelWPInstall($path);
            $i->initData($idata);
            $wpInstalls[$path] = $i;
        }

        return $wpInstalls;
    }

    /**
     *
     * @since 2.0
     *
     * @param string  $dataFile
     * @param string  $dataFileVer
     * @return boolean
     */
    public static function updateDataFile( $dataFile, $dataFileVer )
    {
        $args = array( 'dataFile' => $dataFile, 'dataFileVer' => $dataFileVer );

        return DAEscalate::escalate(true, DAEscalate::MGR_CMD_UPDATE_DATA_FILE,
                $args);
    }

    /**
     *
     * @since 2.0
     *
     * @param string       $action
     * @param null|string  $list
     * @param string[]     $extraArgs
     * @return string[]
     */
    public function doAction( $action, $list, $extraArgs = array() )
    {
        $args = array (
            'action' => $action,
            'list' => $list,
            'extraArgs' => $extraArgs
        );

        $ret = DAEscalate::escalate(true, DAEscalate::MGR_CMD_DO_ACTION, $args);

        /**
         * Re-populate $this->wpInstalls
         */
        $this->error = $this->init();

        $finishedList = $ret['finishedList'];
        $workingQueue = $ret['workingQueue'];
        $invalidPathErrs = $ret['invalidPathErrs'];

        if ( !empty($workingQueue) ) {
            $this->workingQueue =
                    array_merge($this->workingQueue, $workingQueue);
        }

        if ( !empty($invalidPathErrs) ) {

            foreach( $invalidPathErrs as $errMsg ) {
                Logger::uiError($errMsg);
            }
        }

        if ( $action == 'scan' ) {
            /**
             * Explicitly clear any data file errors after scanning in case of
             * multiple actions performed in the same process (cli).
             */
            $this->error = 0;
        }

        return $finishedList;
    }

}
